@@ -14,6 +14,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     : Eric Berberich <eric@mpi-inf.mpg.de>
@@ -25,13 +26,15 @@
 #ifndef CGAL_ACK_CURVE_PAIR_ANALYSIS_H
 #define CGAL_ACK_CURVE_PAIR_ANALYSIS_H 1
 
+#include <CGAL/disable_warnings.h>
+
 #include <vector>
 #include <algorithm>
 
 #include <boost/optional.hpp>
 
 #include <CGAL/Handle_with_policy.h>
-#include <boost/iterator/transform_iterator.hpp>
+#include <CGAL/boost/iterator/transform_iterator.hpp>
 
 #include <CGAL/Polynomial.h>
 #include <CGAL/Polynomial_traits_d.h>
@@ -42,10 +45,7 @@
 #include <CGAL/Algebraic_kernel_d/exceptions.h>
 #include <CGAL/Algebraic_kernel_d/Status_line_CPA_1.h>
 
-#if defined(BOOST_MSVC)
-#  pragma warning(push)
-#  pragma warning(disable:4290)
-#endif
+
 
 namespace CGAL {
 
@@ -802,7 +802,7 @@ private:
 	  (typename Polynomial_traits_2::Swap() (sres,0,1),r);
         std::vector<Algebraic_real_1> gcd_roots;
         kernel()->solve_1_object()(gcd,std::back_inserter(gcd_roots),false);
-        int m = gcd_roots.size();
+        size_type m = static_cast<size_type>(gcd_roots.size());
 
         Slice_info slice_info = construct_slice_info(x);
         reduce_number_of_candidates_and_intersections_to
@@ -998,9 +998,9 @@ public:
     void x_to_index(Algebraic_real_1 x, 
                     size_type& idx, bool& event) const {
         const std::vector<Algebraic_real_1>& sl = event_x_coordinates();
-        idx = std::lower_bound(sl.begin(),
-                               sl.end(),
-                               x) - sl.begin();
+        idx = static_cast<size_type>(std::lower_bound(sl.begin(),
+                                                      sl.end(),
+                                                      x) - sl.begin());
         event = (idx < static_cast<size_type>(sl.size()) && (sl[idx] == x));
 
     }
@@ -1122,16 +1122,14 @@ private:
         }
         Intersection_info_container& old_info_container
             = *(this->ptr()->intersection_info_container);
-        size_type n = old_info_container.size();
-        CGAL_assertion(n == static_cast<size_type>
-                              ( new_info_container.size()));
+        std::size_t n = old_info_container.size();
+        CGAL_assertion(n == new_info_container.size());
         //iterate through the vector and update 
         // (-1 stands for "multiplicity unknown")
-        for(size_type i=0;i<n;i++) {
-            size_type m = old_info_container[i].size();
-            CGAL_assertion(m == static_cast<size_type>\
-                                  (new_info_container[i].size()));
-            for(size_type j=0;j<m;j++) {
+        for(std::size_t i=0;i<n;i++) {
+          std::size_t m = old_info_container[i].size();
+            CGAL_assertion(m == new_info_container[i].size());
+            for(std::size_t j=0;j<m;j++) {
                 old_info_container[i][j].mult
                   = (std::max)(new_info_container[i][j].mult,
                                old_info_container[i][j].mult);
@@ -2643,10 +2641,6 @@ reduce_number_of_candidates_and_intersections_to(size_type n,
 
 } //namespace CGAL
 
-
-#if defined(BOOST_MSVC)
-#  pragma warning(pop)
-#endif
-
+#include <CGAL/enable_warnings.h>
 
 #endif
