@@ -13,12 +13,15 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: LGPL-3.0+
 //
 // Author: Luis Peñaranda <luis.penaranda@gmx.com>
 
 #ifndef CGAL_GMPFR_TYPE_H
 #define CGAL_GMPFR_TYPE_H
 
+#include <CGAL/disable_warnings.h>
+
 #include <CGAL/gmp.h>
 #include <mpfr.h>
 #include <boost/operators.hpp>
@@ -348,7 +351,10 @@ class Gmpfr:
         // only avoid the binary incompatibility of a CGAL program compiled
         // with MSVC with the libmpfr-1.dll compiled with mingw.
 #ifdef _MSC_VER
+#  pragma warning(push)
+#  pragma warning(disable: 4244)
         CGAL_GMPFR_CONSTRUCTOR_FROM_TYPE(long double,mpfr_set_d);
+#  pragma warning(pop)  
 #else
         CGAL_GMPFR_CONSTRUCTOR_FROM_TYPE(long double,mpfr_set_ld);
 #endif
@@ -885,7 +891,11 @@ CGAL_GMPFR_ARITHMETIC_FUNCTION(cbrt,mpfr_cbrt)
 inline
 Gmpfr Gmpfr::kthroot(int k,std::float_round_style r)const{
         Gmpfr result(0,CGAL_GMPFR_MEMBER_PREC());
-        mpfr_root(result.fr(),fr(),k,_gmp_rnd(r));
+        #if(MPFR_VERSION_MAJOR < 4)
+            mpfr_root(result.fr(),fr(),k,_gmp_rnd(r));
+        #else
+            mpfr_rootn_ui(result.fr(),fr(),k,_gmp_rnd(r));
+        #endif
         return result;
 }
 
@@ -895,7 +905,11 @@ Gmpfr Gmpfr::kthroot(int k,
                      std::float_round_style r)const{
         CGAL_assertion(p>=MPFR_PREC_MIN&&p<=MPFR_PREC_MAX);
         Gmpfr result(0,p);
-        mpfr_root(result.fr(),fr(),k,_gmp_rnd(r));
+        #if(MPFR_VERSION_MAJOR < 4)
+            mpfr_root(result.fr(),fr(),k,_gmp_rnd(r));
+        #else
+            mpfr_rootn_ui(result.fr(),fr(),k,_gmp_rnd(r));
+        #endif
         return result;
 }
 
@@ -1282,17 +1296,17 @@ bool operator==(const Gmpfr &a,double b){
 #ifdef _MSC_VER
 inline
 bool operator<(const Gmpfr &a,long double b){
-        return(mpfr_cmp_d(a.fr(),b)<0);
+        return(mpfr_cmp_d(a.fr(),static_cast<double>(b))<0);
 }
 
 inline
 bool operator>(const Gmpfr &a,long double b){
-        return(mpfr_cmp_d(a.fr(),b)>0);
+        return(mpfr_cmp_d(a.fr(),static_cast<double>(b))>0);
 }
 
 inline
 bool operator==(const Gmpfr &a,long double b){
-        return !mpfr_cmp_d(a.fr(),b);
+        return !mpfr_cmp_d(a.fr(),static_cast<double>(b));
 }
 #else
 inline
@@ -1338,4 +1352,6 @@ Gmpfr max BOOST_PREVENT_MACRO_SUBSTITUTION(const Gmpfr& x,const Gmpfr& y){
 
 } // namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif  // CGAL_GMPFR_TYPE_H
