@@ -37,6 +37,7 @@
 #include <tbb/concurrent_vector.h>
 #include <tbb/scalable_allocator.h>
 
+#include <tbb/atomic.h>
 
 #include <vector>
 
@@ -272,6 +273,8 @@ class WorkItem
 {
 public:
   WorkItem() {}
+  virtual ~WorkItem() { }
+
   // Derived class defines the actual work.
   virtual void run() = 0;
   virtual bool less_than(const WorkItem &) const = 0;
@@ -300,12 +303,15 @@ public:
     : m_func(func), m_quality(quality)
   {}
 
+  virtual ~MeshRefinementWorkItem()
+  {}
+
   void run()
   {
     m_func();
     tbb::scalable_allocator<MeshRefinementWorkItem<Func, Quality> >().deallocate(this, 1);
   }
-  
+
   bool less_than (const WorkItem &other) const
   {
     /*try
@@ -349,12 +355,12 @@ public:
     m_func();
     tbb::scalable_allocator<SimpleFunctorWorkItem<Func> >().deallocate(this, 1);
   }
-  
+
   // Irrelevant here
-  bool less_than (const WorkItem &other) const 
-  { 
+  bool less_than (const WorkItem &other) const
+  {
     // Just compare addresses
-    return this < &other; 
+    return this < &other;
   }
 
 private:
@@ -449,14 +455,14 @@ public:
   template <typename Func>
   void enqueue_work(Func f, tbb::task &parent_task) const
   {
-    WorkItem *p_item = 
+    WorkItem *p_item =
       tbb::scalable_allocator<SimpleFunctorWorkItem<Func> >().allocate(1);
     new (p_item) SimpleFunctorWorkItem<Func>(f);
     enqueue_task(create_task(p_item, parent_task));
   }
-  
+
 protected:
-  
+
   WorkItemTask *create_task(WorkItem *pwi, tbb::task &parent_task) const
   {
     return new(tbb::task::allocate_additional_child_of(parent_task)) WorkItemTask(pwi);
@@ -721,6 +727,7 @@ public:
         Concurrent_mesher_config::get().num_work_items_per_batch)
   {
     set_bbox(bbox);
+    m_cache_number_of_tasks = 0;
   }
 
   /// Destructor
@@ -732,12 +739,12 @@ public:
   {
     // We don't need it.
   }
-  
+
   template <typename Func>
   void enqueue_work(Func f, tbb::task &parent_task)
   {
     //WorkItem *p_item = new SimpleFunctorWorkItem<Func>(f);
-    WorkItem *p_item = 
+    WorkItem *p_item =
       tbb::scalable_allocator<SimpleFunctorWorkItem<Func> >().allocate(1);
     new (p_item) SimpleFunctorWorkItem<Func>(f);
     WorkBatch &workbuffer = m_tls_work_buffers.local();
@@ -747,12 +754,13 @@ public:
       add_batch_and_enqueue_task(workbuffer, parent_task);
       workbuffer.clear();
     }
+    m_cache_number_of_tasks = parent_task.ref_count();
   }
 
   template <typename Func, typename Quality>
   void enqueue_work(Func f, const Quality &quality, tbb::task &parent_task)
   {
-    WorkItem *p_item = 
+    WorkItem *p_item =
       tbb::scalable_allocator<MeshRefinementWorkItem<Func, Quality> >()
       .allocate(1);
     new (p_item) MeshRefinementWorkItem<Func, Quality>(f, quality);
@@ -763,6 +771,7 @@ public:
       add_batch_and_enqueue_task(workbuffer, parent_task);
       workbuffer.clear();
     }
+    m_cache_number_of_tasks = parent_task.ref_count();
   }
 
   // Returns true if some items were flushed
@@ -790,9 +799,14 @@ public:
       enqueue_task(*it, parent_task);
     }
 
+    m_cache_number_of_tasks = parent_task.ref_count();
     return (num_flushed_items > 0);
   }
 
+  int approximate_number_of_enqueued_element() const {
+    return int(m_cache_number_of_tasks) * int(NUM_WORK_ITEMS_PER_BATCH);
+  }
+
 protected:
 
   // TLS
@@ -817,6 +831,7 @@ protected:
   }
 
   const size_t                      NUM_WORK_ITEMS_PER_BATCH;
+  tbb::atomic<int>                  m_cache_number_of_tasks;
   TLS_WorkBuffer                    m_tls_work_buffers;
 };
 
