@@ -14,6 +14,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Sebastien Loriot
@@ -35,29 +36,44 @@
 
 #include <boost/dynamic_bitset.hpp>
 
-
-#define CGAL_COREF_SELECT_OUT_ECM(I) \
-  (I == 0 ? cpp11::get<0>(out_edge_mark_maps) \
-          : I == 1 ? cpp11::get<1>(out_edge_mark_maps) \
-                   : I == 2 ? cpp11::get<2>(out_edge_mark_maps) \
-                            : cpp11::get<3>(out_edge_mark_maps))
+// required to handle the multiple types of edge constrained maps
+// for the different output types. CGAL_COREF_FUNCTION_CALL_DEF
+// must be defined prior to using this macro.
+#define CGAL_COREF_FUNCTION_CALL(BO_type) \
+  switch(BO_type) \
+  { \
+    case UNION: \
+      CGAL_COREF_FUNCTION_CALL_DEF(UNION); \
+      break; \
+    case INTERSECTION: \
+      CGAL_COREF_FUNCTION_CALL_DEF(INTERSECTION); \
+      break; \
+    case TM1_MINUS_TM2: \
+      CGAL_COREF_FUNCTION_CALL_DEF(TM1_MINUS_TM2); \
+      break; \
+    default: \
+      CGAL_assertion( BO_type == TM2_MINUS_TM1 ); \
+      CGAL_COREF_FUNCTION_CALL_DEF(TM2_MINUS_TM1); \
+  }
 
 namespace CGAL {
+namespace Polygon_mesh_processing {
 namespace Corefinement {
 
-enum Boolean_operation {UNION = 0, INTER,
-                        TM1_MINUS_TM2, TM2_MINUS_TM1,
-                        NONE };
+enum Boolean_operation_type {UNION = 0, INTERSECTION,
+                             TM1_MINUS_TM2, TM2_MINUS_TM1, NONE };
 
 namespace PMP=Polygon_mesh_processing;
 namespace params=PMP::parameters;
 
 template <class TriangleMesh,
           class VertexPointMap,
+          class VpmOutTuple,
           class FaceIdMap,
           class Kernel_=Default,
-          class EdgeMarkMapBind_=Default,
-          class EdgeMarkMapTuple_=Default >
+          class EdgeMarkMapBind_  = Default,
+          class EdgeMarkMapTuple_ = Default,
+          class UserVisitor_      = Default>
 class Face_graph_output_builder
 {
 //Default typedefs
@@ -74,6 +90,8 @@ class Face_graph_output_builder
                   No_mark<TriangleMesh>,
                   No_mark<TriangleMesh>,
                   No_mark<TriangleMesh> > >::type     EdgeMarkMapTuple;
+  typedef typename Default::Get<
+    UserVisitor_, Default_visitor<TriangleMesh> >::type  UserVisitor;
 
 // graph_traits typedefs
   typedef TriangleMesh                                              TM;
@@ -87,11 +105,15 @@ class Face_graph_output_builder
   typedef std::pair<Node_id,Node_id>                      Node_id_pair;
   typedef boost::unordered_map<edge_descriptor,
                                Node_id_pair >    Intersection_edge_map;
+  // to maintain a halfedge on each polyline per TriangleMesh + pair<bool,size_t>
+  // with first = "is the key (pair<Node_id,Node_id>) was reversed?" and
+  // second is the number of edges -1 in the polyline
   typedef std::map< Node_id_pair,
                     std::pair< std::map<TriangleMesh*,
                                         halfedge_descriptor>,
                                std::pair<bool,std::size_t> > >
                                               An_edge_per_polyline_map;
+
   typedef boost::unordered_map<vertex_descriptor, Node_id> Node_id_map;
   typedef boost::unordered_map<edge_descriptor,
                                edge_descriptor>               Edge_map;
@@ -102,10 +124,11 @@ class Face_graph_output_builder
   const FaceIdMap &fids1, &fids2;
   EdgeMarkMapBind& marks_on_input_edges;
   // property maps of output meshes
-  const cpp11::array<VertexPointMap*, 4 >& output_vpms;
+  const VpmOutTuple& output_vpms;
   EdgeMarkMapTuple& out_edge_mark_maps;
+  UserVisitor& user_visitor;
   // output meshes
-  const cpp11::array<boost::optional<TriangleMesh*>, 4>& desired_output;
+  const cpp11::array<boost::optional<TriangleMesh*>, 4>& requested_output;
   // input meshes closed ?
   /// \todo do we really need this?
   bool is_tm1_closed;
@@ -122,6 +145,22 @@ class Face_graph_output_builder
   // 2 = tm1 - tm2
   // 3 = tm2 - tm1
   std::bitset<4> impossible_operation;
+  // for mapping an edge per polyline per triangle mesh
+  An_edge_per_polyline_map an_edge_per_polyline;
+  // To collect all intersection edges
+  class Mesh_to_intersection_edges{
+    TriangleMesh& m_tm;
+    Intersection_edge_map tm_map;
+    Intersection_edge_map other_map;
+  public:
+    Mesh_to_intersection_edges(TriangleMesh& tm1, TriangleMesh) : m_tm(tm1) {}
+    Intersection_edge_map& operator[](TriangleMesh* tm_ptr) {
+      return &m_tm==tm_ptr?tm_map:other_map;
+    }
+  };
+  Mesh_to_intersection_edges mesh_to_intersection_edges;
+
+  typename An_edge_per_polyline_map::iterator last_polyline;
 
   Node_id get_node_id(vertex_descriptor v,
                       const Node_id_map& node_ids)
@@ -230,14 +269,14 @@ class Face_graph_output_builder
 
   template<class EdgeMarkMap>
   void mark_edges(const EdgeMarkMap& edge_mark_map,
-                 const std::vector<edge_descriptor>& edges)
+                  const std::vector<edge_descriptor>& edges)
   {
     BOOST_FOREACH(edge_descriptor ed, edges)
       put(edge_mark_map, ed, true);
   }
 
   void mark_edges(const No_mark<TriangleMesh>&,
-                 const std::vector<edge_descriptor>&)
+                  const std::vector<edge_descriptor>&)
   {} //nothing to do
 
   template<class EdgeMarkMapTuple>
@@ -315,22 +354,25 @@ public:
                             const FaceIdMap& fids1,
                             const FaceIdMap& fids2,
                                   EdgeMarkMapBind& marks_on_input_edges,
-                            const cpp11::array<VertexPointMap*, 4>& output_vpms,
+                            const VpmOutTuple& output_vpms,
                                   EdgeMarkMapTuple& out_edge_mark_maps,
+                                  UserVisitor& user_visitor,
                             const cpp11::array<
-                              boost::optional<TriangleMesh*>, 4 >& desired_output)
+                              boost::optional<TriangleMesh*>, 4 >& requested_output)
     : tm1(tm1), tm2(tm2)
     , vpm1(vpm1), vpm2(vpm2)
     , fids1(fids1), fids2(fids2)
     , marks_on_input_edges(marks_on_input_edges)
     , output_vpms(output_vpms)
     , out_edge_mark_maps(out_edge_mark_maps)
-    , desired_output(desired_output)
+    , user_visitor(user_visitor)
+    , requested_output(requested_output)
     , is_tm1_closed( is_closed(tm1))
     , is_tm2_closed( is_closed(tm2))
     , is_tm1_inside_out( is_tm1_closed && !PMP::is_outward_oriented(tm1) )
     , is_tm2_inside_out( is_tm2_closed && !PMP::is_outward_oriented(tm2) )
-    , NID(-1)
+    , NID((std::numeric_limits<Node_id>::max)())
+    , mesh_to_intersection_edges(tm1, tm2)
   {}
 
   bool union_is_valid() const
@@ -339,7 +381,7 @@ public:
   }
   bool intersection_is_valid() const
   {
-    return !impossible_operation[INTER];
+    return !impossible_operation[INTERSECTION];
   }
   bool tm1_minus_tm2_is_valid() const
   {
@@ -349,13 +391,53 @@ public:
   {
     return !impossible_operation[TM2_MINUS_TM1];
   }
+// functions called by the intersection visitor
+  void start_new_polyline(Node_id i, Node_id j)
+  {
+    std::pair<typename An_edge_per_polyline_map::iterator,bool> res=
+      an_edge_per_polyline.insert(
+        std::make_pair( make_sorted_pair(i,j),
+          std::make_pair( std::map<TriangleMesh*,halfedge_descriptor>(),std::make_pair(false,0))  )
+      );
+    CGAL_assertion(res.second);
+    last_polyline=res.first;
+    if ( i !=last_polyline->first.first )
+      last_polyline->second.second.first=true;
+  }
+
+  void add_node_to_polyline(Node_id)
+  {
+    ++(last_polyline->second.second.second);
+  }
+
+  void set_edge_per_polyline(TriangleMesh& tm,
+                             Node_id_pair indices,
+                             halfedge_descriptor hedge)
+  {
+    //register an intersection halfedge
+    // It is important here not to use operator[] since a two edges might be
+    // equals while the indices are reversed
+    mesh_to_intersection_edges[&tm].
+      insert(std::make_pair(edge(hedge, tm), indices));
+
+    if (indices.first>indices.second)
+    {
+      std::swap(indices.first,indices.second);
+      hedge=opposite(hedge,tm);
+    }
+    typename An_edge_per_polyline_map::iterator it =
+      an_edge_per_polyline.find(indices);
+
+    if (it!=an_edge_per_polyline.end()){
+      CGAL_assertion(it->second.first.count(&tm) == 0 ||
+                     it->second.first[&tm]==hedge);
+      it->second.first.insert( std::make_pair( &tm,hedge) );
+    }
+  }
 
   template <class Nodes_vector, class Mesh_to_map_node>
   void operator()(
-    std::map<const TriangleMesh*,
-             Intersection_edge_map>& mesh_to_intersection_edges,
     const Nodes_vector& nodes,
-    An_edge_per_polyline_map& an_edge_per_polyline,
     bool input_have_coplanar_faces,
     const boost::dynamic_bitset<>& is_node_of_degree_one,
     const Mesh_to_map_node&)
@@ -393,7 +475,7 @@ public:
     boost::dynamic_bitset<> tm1_coplanar_faces(num_faces(tm1), 0);
     boost::dynamic_bitset<> tm2_coplanar_faces(num_faces(tm2), 0);
 
-    // In the following loop we filter intersection edge that are strictly inside a patch
+    // In the following loop we filter intersection edges that are strictly inside a patch
     // of coplanar facets so that we keep only the edges on the border of the patch.
     // This is not optimal and in an ideal world being able to find the outside edges
     // directly would avoid to compute the intersection of edge/facets inside the patch
@@ -519,12 +601,14 @@ public:
     // (2-a) Use the orientation around an edge to classify a patch
     boost::dynamic_bitset<> is_patch_inside_tm2(nb_patches_tm1, false);
     boost::dynamic_bitset<> is_patch_inside_tm1(nb_patches_tm2, false);
-    boost::dynamic_bitset<> patch_status_not_set_tm1(nb_patches_tm1,true);
-    boost::dynamic_bitset<> patch_status_not_set_tm2(nb_patches_tm2,true);
+    boost::dynamic_bitset<> patch_status_not_set_tm1(nb_patches_tm1);
+    boost::dynamic_bitset<> patch_status_not_set_tm2(nb_patches_tm2);
     boost::dynamic_bitset<> coplanar_patches_of_tm1(nb_patches_tm1,false);
     boost::dynamic_bitset<> coplanar_patches_of_tm2(nb_patches_tm2,false);
     boost::dynamic_bitset<> coplanar_patches_of_tm1_for_union_and_intersection(nb_patches_tm1,false);
     boost::dynamic_bitset<> coplanar_patches_of_tm2_for_union_and_intersection(nb_patches_tm2,false);
+    patch_status_not_set_tm1.set();
+    patch_status_not_set_tm2.set();
 
     for (typename An_edge_per_polyline_map::iterator
             it=an_edge_per_polyline.begin(),
@@ -665,7 +749,7 @@ public:
                    vpm1, vpm2,
                    nodes) ) //p1==q2
             {
-              CGAL_assertion( index_p1!=index_p2 || index_p1==Node_id(-1) );
+              CGAL_assertion( index_p1!=index_p2 || index_p1==Node_id((std::numeric_limits<Node_id>::max)()) );
               coplanar_patches_of_tm1.set(patch_id_p1);
               coplanar_patches_of_tm2.set(patch_id_q2);
               bool q1_is_between_p1p2 = sorted_around_edge(
@@ -735,17 +819,17 @@ public:
 #endif //CGAL_COREFINEMENT_POLYHEDRA_DEBUG
 
           CGAL_assertion(
-              ( index_p1 == Node_id(-1) ? nodes.to_exact(get(vpm1,p1)): nodes.exact_node(index_p1) ) !=
-              ( index_q1 == Node_id(-1) ? nodes.to_exact(get(vpm2,q1)): nodes.exact_node(index_q1) )
+              ( index_p1 == Node_id((std::numeric_limits<Node_id>::max)()) ? nodes.to_exact(get(vpm1,p1)): nodes.exact_node(index_p1) ) !=
+              ( index_q1 == Node_id((std::numeric_limits<Node_id>::max)()) ? nodes.to_exact(get(vpm2,q1)): nodes.exact_node(index_q1) )
           &&
-              ( index_p2 == Node_id(-1) ? nodes.to_exact(get(vpm1,p2)): nodes.exact_node(index_p2) ) !=
-              ( index_q1 == Node_id(-1) ? nodes.to_exact(get(vpm2,q1)): nodes.exact_node(index_q1) )
+              ( index_p2 == Node_id((std::numeric_limits<Node_id>::max)()) ? nodes.to_exact(get(vpm1,p2)): nodes.exact_node(index_p2) ) !=
+              ( index_q1 == Node_id((std::numeric_limits<Node_id>::max)()) ? nodes.to_exact(get(vpm2,q1)): nodes.exact_node(index_q1) )
           &&
-              ( index_p1 == Node_id(-1) ? nodes.to_exact(get(vpm1,p1)): nodes.exact_node(index_p1) ) !=
-              ( index_q2 == Node_id(-1) ? nodes.to_exact(get(vpm2,q2)): nodes.exact_node(index_q2) )
+              ( index_p1 == Node_id((std::numeric_limits<Node_id>::max)()) ? nodes.to_exact(get(vpm1,p1)): nodes.exact_node(index_p1) ) !=
+              ( index_q2 == Node_id((std::numeric_limits<Node_id>::max)()) ? nodes.to_exact(get(vpm2,q2)): nodes.exact_node(index_q2) )
           &&
-              ( index_p2 == Node_id(-1) ? nodes.to_exact(get(vpm1,p2)): nodes.exact_node(index_p2) ) !=
-              ( index_q2 == Node_id(-1) ? nodes.to_exact(get(vpm2,q2)): nodes.exact_node(index_q2) )
+              ( index_p2 == Node_id((std::numeric_limits<Node_id>::max)()) ? nodes.to_exact(get(vpm1,p2)): nodes.exact_node(index_p2) ) !=
+              ( index_q2 == Node_id((std::numeric_limits<Node_id>::max)()) ? nodes.to_exact(get(vpm2,q2)): nodes.exact_node(index_q2) )
           );
 
           bool q1_is_between_p1p2 = sorted_around_edge(
@@ -788,7 +872,7 @@ public:
                 // opposite( poly_first U poly_second ) = {O}
                 is_patch_inside_tm2.set(patch_id_p1);
                 is_patch_inside_tm2.set(patch_id_p2);
-                impossible_operation.set(INTER); // tm1 n tm2 is non-manifold
+                impossible_operation.set(INTERSECTION); // tm1 n tm2 is non-manifold
               }
             }
             else
@@ -967,7 +1051,7 @@ public:
 
     //to maintain a halfedge on each polyline + pair<bool,int>
     //with first = "is the key (pair<Node_id,Node_id>) was reversed?"
-    // and second is the number of edges +1 in the polyline
+    // and second is the number of edges -1 in the polyline
     //typedef std::map< std::pair<Node_id,Node_id>,
     //                  std::pair< std::map<TriangleMesh*,
     //                                      halfedge_descriptor>,
@@ -1039,7 +1123,7 @@ public:
     std::vector< boost::dynamic_bitset<> > patches_of_tm2_used(4);
 
     /// handle the bitset for the union
-    if ( !impossible_operation.test(UNION) && desired_output[UNION] )
+    if ( !impossible_operation.test(UNION) && requested_output[UNION] )
     {
       //define patches to import from P
       patches_of_tm1_used[UNION] = ~is_patch_inside_tm2 - coplanar_patches_of_tm1;
@@ -1048,7 +1132,7 @@ public:
       //handle coplanar patches
       if (coplanar_patches_of_tm1.any())
       {
-        if (desired_output[UNION]==&tm2)
+        if (requested_output[UNION]==&tm2)
           patches_of_tm2_used[UNION] |= coplanar_patches_of_tm2_for_union_and_intersection;
         else
           patches_of_tm1_used[UNION] |= coplanar_patches_of_tm1_for_union_and_intersection;
@@ -1056,24 +1140,24 @@ public:
     }
 
     /// handle the bitset for the intersection
-    if ( !impossible_operation.test(INTER) && desired_output[INTER] )
+    if ( !impossible_operation.test(INTERSECTION) && requested_output[INTERSECTION] )
     {
       //define patches to import from P
-      patches_of_tm1_used[INTER] = is_patch_inside_tm2;
+      patches_of_tm1_used[INTERSECTION] = is_patch_inside_tm2;
       //define patches to import from Q
-      patches_of_tm2_used[INTER] = is_patch_inside_tm1;
+      patches_of_tm2_used[INTERSECTION] = is_patch_inside_tm1;
       //handle coplanar patches
       if (coplanar_patches_of_tm1.any())
       {
-        if (desired_output[INTER]==&tm2)
-          patches_of_tm2_used[INTER] |= coplanar_patches_of_tm2_for_union_and_intersection;
+        if (requested_output[INTERSECTION]==&tm2)
+          patches_of_tm2_used[INTERSECTION] |= coplanar_patches_of_tm2_for_union_and_intersection;
         else
-          patches_of_tm1_used[INTER] |= coplanar_patches_of_tm1_for_union_and_intersection;
+          patches_of_tm1_used[INTERSECTION] |= coplanar_patches_of_tm1_for_union_and_intersection;
       }
     }
 
     /// handle the bitset for P-Q
-    if ( !impossible_operation.test(TM1_MINUS_TM2) && desired_output[TM1_MINUS_TM2] )
+    if ( !impossible_operation.test(TM1_MINUS_TM2) && requested_output[TM1_MINUS_TM2] )
     {
       //define patches to import from P
       patches_of_tm1_used[TM1_MINUS_TM2] = (~is_patch_inside_tm2 - coplanar_patches_of_tm1);
@@ -1082,7 +1166,7 @@ public:
       //handle coplanar patches
       if (coplanar_patches_of_tm1.any())
       {
-        if (desired_output[TM1_MINUS_TM2]==&tm2)
+        if (requested_output[TM1_MINUS_TM2]==&tm2)
           patches_of_tm2_used[TM1_MINUS_TM2] |= ~coplanar_patches_of_tm2_for_union_and_intersection & coplanar_patches_of_tm2;
         else
           patches_of_tm1_used[TM1_MINUS_TM2] |= ~coplanar_patches_of_tm1_for_union_and_intersection & coplanar_patches_of_tm1;
@@ -1090,7 +1174,7 @@ public:
     }
 
     /// handle the bitset for Q-P
-    if ( !impossible_operation.test(TM2_MINUS_TM1) && desired_output[TM2_MINUS_TM1] )
+    if ( !impossible_operation.test(TM2_MINUS_TM1) && requested_output[TM2_MINUS_TM1] )
     {
       //define patches to import from P
       patches_of_tm1_used[TM2_MINUS_TM1] = is_patch_inside_tm2;
@@ -1099,7 +1183,7 @@ public:
       //handle coplanar patches
       if (coplanar_patches_of_tm1.any())
       {
-        if (desired_output[TM2_MINUS_TM1]==&tm2)
+        if (requested_output[TM2_MINUS_TM1]==&tm2)
           patches_of_tm2_used[TM2_MINUS_TM1] |= ~coplanar_patches_of_tm2_for_union_and_intersection & coplanar_patches_of_tm2;
         else
           patches_of_tm1_used[TM2_MINUS_TM1] |= ~coplanar_patches_of_tm1_for_union_and_intersection & coplanar_patches_of_tm1;
@@ -1110,8 +1194,8 @@ public:
     #ifdef CGAL_COREFINEMENT_DEBUG
     std::cout << "patches_of_tm1_used[UNION] " << patches_of_tm1_used[UNION] << "\n";
     std::cout << "patches_of_tm2_used[UNION] " << patches_of_tm2_used[UNION] << "\n";
-    std::cout << "patches_of_tm1_used[INTER] " << patches_of_tm1_used[INTER] << "\n";
-    std::cout << "patches_of_tm2_used[INTER] " << patches_of_tm2_used[INTER] << "\n";
+    std::cout << "patches_of_tm1_used[INTERSECTION] " << patches_of_tm1_used[INTERSECTION] << "\n";
+    std::cout << "patches_of_tm2_used[INTERSECTION] " << patches_of_tm2_used[INTERSECTION] << "\n";
     std::cout << "patches_of_tm1_used[TM1_MINUS_TM2] " << patches_of_tm1_used[TM1_MINUS_TM2] << "\n";
     std::cout << "patches_of_tm2_used[TM1_MINUS_TM2] " << patches_of_tm2_used[TM1_MINUS_TM2] << "\n";
     std::cout << "patches_of_tm1_used[TM2_MINUS_TM1] " << patches_of_tm1_used[TM2_MINUS_TM1] << "\n";
@@ -1121,29 +1205,29 @@ public:
     // should be done. First operations are those filling meshes
     // different from tm1 and tm2, then the one modifying tm1 and
     // finally the one modifying tm2.
-    std::vector<Boolean_operation> out_of_place_operations;
-    Boolean_operation inplace_operation_tm1=NONE,
-                      inplace_operation_tm2=NONE;
+    std::vector<Boolean_operation_type> out_of_place_operations;
+    Boolean_operation_type inplace_operation_tm1=NONE,
+                           inplace_operation_tm2=NONE;
     for (int i=0;i<4;++i)
     {
-      Boolean_operation operation=enum_cast<Boolean_operation>(i);
+      Boolean_operation_type operation=enum_cast<Boolean_operation_type>(i);
 
-      if (!desired_output[operation] || impossible_operation.test(operation))
+      if (!requested_output[operation] || impossible_operation.test(operation))
         continue;
 
-      if (desired_output[operation]==&tm1)
+      if (requested_output[operation]==&tm1)
         inplace_operation_tm1=operation;
       else
-        if (desired_output[operation]==&tm2)
+        if (requested_output[operation]==&tm2)
           inplace_operation_tm2=operation;
         else
           out_of_place_operations.push_back(operation);
     }
 
     /// first handle operations in a mesh that is neither tm1 nor tm2
-    BOOST_FOREACH(Boolean_operation operation, out_of_place_operations)
+    BOOST_FOREACH(Boolean_operation_type operation, out_of_place_operations)
     {
-      TriangleMesh& output = *(*desired_output[operation]);
+      TriangleMesh& output = *(*requested_output[operation]);
       CGAL_assertion(&tm1!=&output && &tm2!=&output);
 
       Intersection_polylines polylines(tm1_polylines,
@@ -1158,19 +1242,24 @@ public:
       );
 
       std::vector<edge_descriptor> shared_edges;
-      fill_new_triangle_mesh(
-        output,
-        patches_of_tm1_used[operation], patches_of_tm2_used[operation],
-        patches_of_tm1, patches_of_tm2,
-        operation == TM2_MINUS_TM1, operation == TM1_MINUS_TM2,
-        polylines,
-        intersection_edges1, intersection_edges2,
-        vpm1, vpm2, *output_vpms[operation],
-        marks_on_input_edges.ecm1,
-        marks_on_input_edges.ecm2,
-        CGAL_COREF_SELECT_OUT_ECM(operation),
-        shared_edges
-      );
+
+      #define CGAL_COREF_FUNCTION_CALL_DEF(BO_type) \
+        fill_new_triangle_mesh( \
+          output, \
+          patches_of_tm1_used[BO_type], patches_of_tm2_used[BO_type], \
+          patches_of_tm1, patches_of_tm2, \
+          BO_type == TM2_MINUS_TM1, BO_type == TM1_MINUS_TM2, \
+          polylines, \
+          intersection_edges1, intersection_edges2, \
+          vpm1, vpm2, *cpp11::get<BO_type>(output_vpms), \
+          marks_on_input_edges.ecm1, \
+          marks_on_input_edges.ecm2, \
+          cpp11::get<BO_type>(out_edge_mark_maps), \
+          shared_edges, \
+          user_visitor \
+        )
+      CGAL_COREF_FUNCTION_CALL(operation)
+      #undef CGAL_COREF_FUNCTION_CALL_DEF
       mark_edges(out_edge_mark_maps, shared_edges, operation);
     }
 
@@ -1184,7 +1273,7 @@ public:
                  mesh_to_intersection_edges[&tm1],
                  inplace_operation_tm1);
 
-      CGAL_assertion( *desired_output[inplace_operation_tm1] == &tm1 );
+      CGAL_assertion( *requested_output[inplace_operation_tm1] == &tm1 );
 
       if ( inplace_operation_tm2!=NONE)
       {
@@ -1235,34 +1324,44 @@ public:
           patches_of_tm1[i];
         }
         // Operation in tm1: disconnect patches not use and append the one from tm2
-        compute_inplace_operation_delay_removal_and_insideout(
-          tm1,
-          tm2,
-          patches_of_tm1_used[inplace_operation_tm1],
-          patches_of_tm2_used[inplace_operation_tm1],
-          patches_of_tm1, patches_of_tm2,
-          inplace_operation_tm1 == TM1_MINUS_TM2 ||
-            inplace_operation_tm1 == TM2_MINUS_TM1,
-          polylines_in_tm1,
-          vpm1, vpm2,
-          marks_on_input_edges.ecm1,
-          marks_on_input_edges.ecm2,
-          CGAL_COREF_SELECT_OUT_ECM(inplace_operation_tm1),
-          disconnected_patches_edge_to_tm2_edge);
+        #define CGAL_COREF_FUNCTION_CALL_DEF(BO_type)\
+        compute_inplace_operation_delay_removal_and_insideout( \
+          tm1, \
+          tm2, \
+          patches_of_tm1_used[BO_type], \
+          patches_of_tm2_used[BO_type], \
+          patches_of_tm1, patches_of_tm2, \
+          BO_type == TM1_MINUS_TM2 || \
+            BO_type == TM2_MINUS_TM1, \
+          polylines_in_tm1, \
+          vpm1, vpm2, \
+          marks_on_input_edges.ecm1, \
+          marks_on_input_edges.ecm2, \
+          cpp11::get<BO_type>(out_edge_mark_maps), \
+          disconnected_patches_edge_to_tm2_edge, \
+          user_visitor)
+        CGAL_COREF_FUNCTION_CALL(inplace_operation_tm1)
+        #undef CGAL_COREF_FUNCTION_CALL_DEF
         // Operation in tm2: discard patches and append the one from tm2
-        CGAL_assertion( *desired_output[inplace_operation_tm2] == &tm2 );
-        compute_inplace_operation( tm2, tm1,
-                                   patches_of_tm2_used[inplace_operation_tm2],
-                                   patches_of_tm1_used[inplace_operation_tm2],
-                                   patches_of_tm2, patches_of_tm1,
-                                   inplace_operation_tm2==TM1_MINUS_TM2,
-                                   inplace_operation_tm2==TM2_MINUS_TM1,
-                                   vpm2,
-                                   vpm1,
-                                   marks_on_input_edges.ecm2,
-                                   marks_on_input_edges.ecm1,
-                                   CGAL_COREF_SELECT_OUT_ECM(inplace_operation_tm2),
-                                   disconnected_patches_edge_to_tm2_edge);
+        CGAL_assertion( *requested_output[inplace_operation_tm2] == &tm2 );
+
+        #define CGAL_COREF_FUNCTION_CALL_DEF(BO_type)\
+          compute_inplace_operation( tm2, tm1, \
+                                     patches_of_tm2_used[BO_type], \
+                                     patches_of_tm1_used[BO_type], \
+                                     patches_of_tm2, patches_of_tm1, \
+                                     BO_type==TM1_MINUS_TM2, \
+                                     BO_type==TM2_MINUS_TM1, \
+                                     vpm2, \
+                                     vpm1, \
+                                     marks_on_input_edges.ecm2, \
+                                     marks_on_input_edges.ecm1, \
+                                     cpp11::get<BO_type>(out_edge_mark_maps), \
+                                     disconnected_patches_edge_to_tm2_edge, \
+                                     user_visitor)
+        CGAL_COREF_FUNCTION_CALL(inplace_operation_tm2)
+        #undef CGAL_COREF_FUNCTION_CALL_DEF
+
         // remove polylines only on the border of patches not kept in tm2
         if (polylines_in_tm2.to_skip.any())
           remove_unused_polylines(tm2,
@@ -1275,8 +1374,12 @@ public:
                                     marks_on_input_edges.ecm1);
 
         // transfer marks of edges of patches kept to the output edge mark property
-        copy_edge_mark<TriangleMesh>(
-          tm1, marks_on_input_edges.ecm1, CGAL_COREF_SELECT_OUT_ECM(inplace_operation_tm1));
+        #define CGAL_COREF_FUNCTION_CALL_DEF(BO_type) \
+          copy_edge_mark<TriangleMesh>( \
+          tm1, marks_on_input_edges.ecm1, \
+          cpp11::get<BO_type>(out_edge_mark_maps))
+        CGAL_COREF_FUNCTION_CALL(inplace_operation_tm1)
+        #undef CGAL_COREF_FUNCTION_CALL_DEF
 
         // remove polylines only on the border of patches not kept in tm1
         if (polylines_in_tm1.to_skip.any())
@@ -1289,7 +1392,7 @@ public:
       }
       else{
         /// handle the operation updating only tm1
-        CGAL_assertion( *desired_output[inplace_operation_tm1] == &tm1 );
+        CGAL_assertion( *requested_output[inplace_operation_tm1] == &tm1 );
         Intersection_polylines polylines(
           tm1_polylines, tm2_polylines, polyline_lengths);
         fill_polylines_to_skip(
@@ -1297,21 +1400,24 @@ public:
           patches_of_tm1_used[inplace_operation_tm1],
           patches_of_tm2_used[inplace_operation_tm1],
           fids1, fids2, tm1, tm2);
-
-        compute_inplace_operation(
-          tm1, tm2,
-          patches_of_tm1_used[inplace_operation_tm1],
-          patches_of_tm2_used[inplace_operation_tm1],
-          patches_of_tm1, patches_of_tm2,
-          inplace_operation_tm1 == TM2_MINUS_TM1,
-          inplace_operation_tm1 == TM1_MINUS_TM2,
-          vpm1,
-          vpm2,
-          marks_on_input_edges.ecm1,
-          marks_on_input_edges.ecm2,
-          CGAL_COREF_SELECT_OUT_ECM(inplace_operation_tm1),
-          polylines
-        );
+        #define CGAL_COREF_FUNCTION_CALL_DEF(BO_type) \
+          compute_inplace_operation( \
+            tm1, tm2, \
+            patches_of_tm1_used[BO_type], \
+            patches_of_tm2_used[BO_type], \
+            patches_of_tm1, patches_of_tm2, \
+            BO_type == TM2_MINUS_TM1, \
+            BO_type == TM1_MINUS_TM2, \
+            vpm1, \
+            vpm2, \
+            marks_on_input_edges.ecm1, \
+            marks_on_input_edges.ecm2, \
+            cpp11::get<BO_type>(out_edge_mark_maps), \
+            polylines, \
+            user_visitor \
+          )
+        CGAL_COREF_FUNCTION_CALL(inplace_operation_tm1)
+        #undef CGAL_COREF_FUNCTION_CALL_DEF
         // remove polylines only on the border of patches not kept
         if (polylines.to_skip.any())
           remove_unused_polylines(tm1,
@@ -1328,7 +1434,7 @@ public:
                    inplace_operation_tm2);
 
         /// handle the operation updating only tm2
-        CGAL_assertion( *desired_output[inplace_operation_tm2] == &tm2 );
+        CGAL_assertion( *requested_output[inplace_operation_tm2] == &tm2 );
         Intersection_polylines polylines(
           tm2_polylines, tm1_polylines, polyline_lengths);
         fill_polylines_to_skip(
@@ -1337,20 +1443,22 @@ public:
           patches_of_tm1_used[inplace_operation_tm2],
           fids2, fids1, tm2, tm1
         );
-
-        compute_inplace_operation( tm2, tm1,
-                                   patches_of_tm2_used[inplace_operation_tm2],
-                                   patches_of_tm1_used[inplace_operation_tm2],
-                                   patches_of_tm2, patches_of_tm1,
-                                   inplace_operation_tm2==TM1_MINUS_TM2,
-                                   inplace_operation_tm2==TM2_MINUS_TM1,
-                                   vpm2,
-                                   vpm1,
-                                   marks_on_input_edges.ecm2,
-                                   marks_on_input_edges.ecm1,
-                                   CGAL_COREF_SELECT_OUT_ECM(inplace_operation_tm2),
-                                   polylines);
-
+        #define CGAL_COREF_FUNCTION_CALL_DEF(BO_type) \
+          compute_inplace_operation( tm2, tm1, \
+                                     patches_of_tm2_used[BO_type], \
+                                     patches_of_tm1_used[BO_type], \
+                                     patches_of_tm2, patches_of_tm1, \
+                                     BO_type==TM1_MINUS_TM2, \
+                                     BO_type==TM2_MINUS_TM1, \
+                                     vpm2, \
+                                     vpm1, \
+                                     marks_on_input_edges.ecm2, \
+                                     marks_on_input_edges.ecm1, \
+                                     cpp11::get<BO_type>(out_edge_mark_maps), \
+                                     polylines, \
+                                     user_visitor);
+        CGAL_COREF_FUNCTION_CALL(inplace_operation_tm2)
+        #undef CGAL_COREF_FUNCTION_CALL_DEF
         // remove polylines only on the border of patches not kept
         if (polylines.to_skip.any())
           remove_unused_polylines(tm2,
@@ -1361,8 +1469,7 @@ public:
 };
 
 
-} } // CGAL::Corefinement
-
-#undef CGAL_COREF_SELECT_OUT_ECM
+} } } // CGAL::Polygon_mesh_processing::Corefinement
 
+#undef CGAL_COREF_FUNCTION_CALL
 #endif // CGAL_POLYGON_MESH_PROCESSING_INTERNAL_FACE_GRAPH_OUTPUT_BUILDER_H
