@@ -14,6 +14,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: GPL-3.0+
 //
 // Author(s)     : Andreas Fabri
 //
@@ -22,6 +23,7 @@
 
 #include <CGAL/license/Polyline_simplification_2.h>
 
+#include <CGAL/disable_warnings.h>
 
 #include <list>
 
@@ -484,4 +486,7 @@ simplify(CGAL::Constrained_triangulation_plus_2<CDT>& ct,
 
 } // namespace polyline_simplification_2
 } // namespace CGAL 
+
+#include <CGAL/enable_warnings.h>
+
 #endif
