@@ -13,6 +13,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: LGPL-3.0+
 //
 // Author(s)     : Laurent Rineau, Xiang Gao
 //
@@ -20,6 +21,8 @@
 #ifndef CGAL_SPLIT_GRAPH_INTO_POLYLINES
 #define CGAL_SPLIT_GRAPH_INTO_POLYLINES
 
+#include <CGAL/disable_warnings.h>
+
 #include <map> 
 #include <vector>
 #include <utility>
@@ -292,7 +295,7 @@ split_graph_into_polylines(const Graph& graph,
   std::set<vertex_descriptor, G_copy_less> terminal(g_copy_less);
 
   BOOST_FOREACH(vertex_descriptor v, vertices(g_copy)){
-    typename graph_traits<Graph>::degree_size_type n = degree(v, g_copy);
+    typename graph_traits<G_copy>::degree_size_type n = degree(v, g_copy);
     if ( n == 1 ) terminal.insert(v);
     if ( n ==0 ){
       //isolated vertex
@@ -374,4 +377,6 @@ split_graph_into_polylines(const Graph& graph,
 
 } //end of namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif //CGAL_SPLIT_GRAPH_INTO_POLYLINES
